@@ -7,6 +7,8 @@ This module provides stubs that are overrideen in pkcs11._pkcs11.
 from threading import RLock
 from binascii import hexlify
 
+from cached_property import cached_property
+
 from .constants import (
     Attribute,
     MechanismFlag,
@@ -28,9 +30,7 @@ from .exceptions import (
 
 def _CK_UTF8CHAR_to_str(data):
     """Convert CK_UTF8CHAR to string."""
-    # FIXME: the last couple of bytes are sometimes bogus, is this me
-    # or SoftHSM?
-    return data[:31].decode('utf-8').rstrip()
+    return data.decode('utf-8').rstrip()
 
 
 def _CK_VERSION_to_tuple(data):
@@ -79,6 +79,12 @@ class MechanismInfo:
             "Flags: %s" % self.flags,
         ))
 
+    def __repr__(self):
+        return '<{klass} (mechanism={mechanism}, flags={flags})>'.format(
+            klass=type(self).__name__,
+            mechanism=str(self.mechanism),
+            flags=str(self.flags))
+
 
 class Slot:
     """
@@ -165,15 +171,29 @@ class Token:
     """
 
     def __init__(self, slot,
-                 label=None, serial=None, flags=None,
+                 label=None,
+                 serialNumber=None,
+                 model=None,
+                 manufacturerID=None,
+                 hardwareVersion=None,
+                 firmwareVersion=None,
+                 flags=None,
                  **kwargs):
 
         self.slot = slot
         """The :class:`Slot` this token is installed in."""
         self.label = _CK_UTF8CHAR_to_str(label)
         """Label of this token (:class:`str`)."""
-        self.serial = serial
+        self.serial = serialNumber.rstrip()
         """Serial number of this token (:class:`bytes`)."""
+        self.manufacturer_id = _CK_UTF8CHAR_to_str(manufacturerID)
+        """Manufacturer ID."""
+        self.model = _CK_UTF8CHAR_to_str(model)
+        """Model name."""
+        self.hardware_version = _CK_VERSION_to_tuple(hardwareVersion)
+        """Hardware version (:class:`tuple`)."""
+        self.firmware_version = _CK_VERSION_to_tuple(firmwareVersion)
+        """Firmware version (:class:`tuple`)."""
         self.flags = TokenFlag(flags)
         """Capabilities of this token (:class:`pkcs11.flags.TokenFlag`)."""
 
@@ -350,7 +370,7 @@ class Session:
 
         For generating keys see :meth:`generate_key` or
         :meth:`generate_keypair`.
-        For importing keys see :meth:`import_key`.
+        For importing keys see :ref:`importing-keys`.
 
         Requires a read/write session, unless the object is not to be
         stored.
@@ -622,7 +642,7 @@ class DomainParameters(Object):
         else:
             super().__setitem__(key, value)
 
-    @property
+    @cached_property
     def key_type(self):
         """
         Key type (:class:`pkcs11.mechanisms.KeyType`) these parameters
@@ -656,28 +676,35 @@ class DomainParameters(Object):
 class Key(Object):
     """Base class for all key :class:`Object` types."""
 
-    @property
+    @cached_property
     def id(self):
         """Key id (:class:`bytes`)."""
         return self[Attribute.ID]
 
-    @property
+    @cached_property
     def label(self):
         """Key label (:class:`str`)."""
         return self[Attribute.LABEL]
 
-    @property
+    @cached_property
     def key_type(self):
         """Key type (:class:`pkcs11.mechanisms.KeyType`)."""
         return self[Attribute.KEY_TYPE]
 
+    @cached_property
+    def _key_description(self):
+        """A description of the key."""
+        try:
+            return '%s-bit %s' % (self.key_length, self.key_type.name)
+        except AttributeTypeInvalid:
+            return self.key_type.name
+
     def __repr__(self):
-        return "<%s label='%s' id='%s' %s-bit %s>" % (
+        return "<%s label='%s' id='%s' %s>" % (
             type(self).__name__,
             self.label,
             hexlify(self.id).decode('ascii'),
-            self.key_length,
-            self.key_type.name)
+            self._key_description)
 
 
 class SecretKey(Key):
@@ -688,7 +715,7 @@ class SecretKey(Key):
 
     object_class = ObjectClass.SECRET_KEY
 
-    @property
+    @cached_property
     def key_length(self):
         """Key length in bits."""
         return self[Attribute.VALUE_LEN] * 8
@@ -706,7 +733,7 @@ class PublicKey(Key):
 
     object_class = ObjectClass.PUBLIC_KEY
 
-    @property
+    @cached_property
     def key_length(self):
         """Key length in bits."""
         return self[Attribute.MODULUS_BITS]
@@ -728,7 +755,7 @@ class PrivateKey(Key):
 
     object_class = ObjectClass.PRIVATE_KEY
 
-    @property
+    @cached_property
     def key_length(self):
         """Key length in bits."""
         return len(self[Attribute.MODULUS]) * 8
@@ -748,7 +775,7 @@ class Certificate(Object):
     """
     object_class = ObjectClass.CERTIFICATE
 
-    @property
+    @cached_property
     def certificate_type(self):
         """
         The type of certificate.
@@ -887,6 +914,12 @@ class SignMixin(Object):
 
         See :meth:`EncryptMixin.encrypt` for more information.
 
+        For DSA and ECDSA keys, PKCS #11 outputs the two parameters (r & s)
+        as two concatenated `biginteger` of the same length. To convert these
+        into other formats, such as the format used by OpenSSL, use
+        :func:`pkcs11.util.dsa.encode_dsa_signature` or
+        :func:`pkcs11.util.ec.encode_ecdsa_signature`.
+
         :param data: data to sign
         :type data: str, bytes or iter(bytes)
         :param Mechanism mechanism: optional signing mechanism
@@ -919,6 +952,12 @@ class VerifyMixin(Object):
 
         Returns True if `signature` is valid for `data`.
 
+        For DSA and ECDSA keys, PKCS #11 expects the two parameters (r & s)
+        as two concatenated `biginteger` of the same length. To convert these
+        from other formats, such as the format used by OpenSSL, use
+        :func:`pkcs11.util.dsa.decode_dsa_signature` or
+        :func:`pkcs11.util.ec.decode_ecdsa_signature`.
+
         :param data: data to sign
         :type data: str, bytes or iter(bytes)
         :param bytes signature: signature
